package cn.wise.sc.consume.business.util;

import java.security.Key;
import java.util.Date;
import java.util.Map;
import java.util.Set;

import javax.crypto.spec.SecretKeySpec;
import javax.xml.bind.DatatypeConverter;

import cn.wise.sc.consume.business.model.LoginUser;
import io.swagger.models.auth.In;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONObject;

import io.jsonwebtoken.Claims;
import io.jsonwebtoken.JwtBuilder;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;
import io.jsonwebtoken.impl.DefaultClaims;

/**
 * 通过 jwt 生成token 及 验证
 *
 * @author wangchuang
 */
public class JwtUtil {

    private static final Logger LOGGER = LoggerFactory.getLogger(JwtUtil.class);


    private static final String SECRET_KEY = "_MeD5CiNov0_TecH";

    //private static final String SECRET_KEY = "XYfbeae_A81cY3eR";

    /**
     * 签名算法
     */
    private static final SignatureAlgorithm DEFAULTSIGN = SignatureAlgorithm.HS256;

    /**
     * 根据 userId 生成 token
     *
     * @param loginUser
     * @return
     */
    public static String createToken(LoginUser loginUser) {
        Claims claim = new DefaultClaims();
        // 设置 签发人
        long timeStamp = (new Date()).getTime();
        claim.setIssuer(loginUser.getTenantId());
        claim.put("id", loginUser.getId());
        claim.put("username", loginUser.getUsername());
        claim.put("nickName", loginUser.getNickName());
        claim.put("phone", loginUser.getPhone());
//        claim.put("bankName", loginUser.getBankName());
//        claim.put("accountNo", loginUser.getAccountNo());
        claim.put("timeStamp", timeStamp);
        claim.putIfAbsent("isAdmin", loginUser.getIsAdmin());
        Key signingKey = getKey(SECRET_KEY, DEFAULTSIGN.getJcaName());
        JwtBuilder builder = Jwts.builder().setClaims(claim).signWith(DEFAULTSIGN, signingKey);
        return builder.compact();
    }

    /**
     * 根据 params 生成 token
     *
     * @return
     */
    public static String createToken(JSONObject params) {
        Claims claim = new DefaultClaims();
        // 设置 签发人
        long timeStamp = (new Date()).getTime();
        claim.setIssuer("goldfoam2");
        Set<String> keys = params.keySet();
        for (String key : keys) {
            claim.put(key, params.getString(key));
        }
        claim.put("timeStamp", timeStamp);
        Key signingKey = getKey(SECRET_KEY, DEFAULTSIGN.getJcaName());
        JwtBuilder builder = Jwts.builder().setClaims(claim).signWith(DEFAULTSIGN, signingKey);
        return builder.compact();
    }
    
    /**
     *  根据 userId 生成 token
     * @param userId
     * @return
     */
    public static String createToken(String userId) {
        Claims claim = new DefaultClaims();
        // claim.setAudience("www.bizconf.cn");// 设置接收方
        // 设置 签发人
        long timeStamp = (new Date()).getTime();
        claim.setIssuer("goldwind");
        claim.put("userId", userId);
        claim.put("timeStamp",timeStamp);
        Key signingKey = getKey(SECRET_KEY, DEFAULTSIGN.getJcaName());
        JwtBuilder builder = Jwts.builder().setClaims(claim).signWith(DEFAULTSIGN, signingKey);
        return builder.compact();
    }

    /**
     * 该方法使用 HS256 算法和 Secret: 生成signKey
     *
     * @param key
     * @param jcaName
     * @return
     */
    private static Key getKey(String key, String jcaName) {
        byte[] keyb = null;
        if ("".equals(key) || key == null) {
            keyb = DatatypeConverter.parseBase64Binary(SECRET_KEY);
        } else {
            keyb = DatatypeConverter.parseBase64Binary(key);
        }
        String jcaName1 = null;
        if ("".equals(jcaName) || jcaName == null) {
            jcaName1 = DEFAULTSIGN.getJcaName();
        } else {
            jcaName1 = jcaName;
        }

        Key signKey = new SecretKeySpec(keyb, jcaName1);
        return signKey;
    }

    /**
     * 解析 token
     *
     * @param token
     * @return
     */
    public static LoginUser parserToken(String token) throws Exception {

        Claims claims = null;
//        String userid = "-1";
        claims = Jwts.parser().setSigningKey(getKey(SECRET_KEY, DEFAULTSIGN.getJcaName())).parseClaimsJws(token)
                .getBody();
		LoginUser loginUser = new LoginUser();
	    loginUser.setId(Long.valueOf(claims.get("id").toString()));
	    loginUser.setUsername(claims.get("username").toString());
	    loginUser.setNickName(claims.get("nickName").toString());
	    loginUser.setPhone(claims.get("phone").toString());
//	    loginUser.setBankName(claims.get("bankName").toString());
//	    loginUser.setAccountNo(claims.get("accountNo").toString());
	    loginUser.setExpireTime(Long.valueOf(claims.get("timeStamp").toString()));
	    loginUser.setIsAdmin(Integer.valueOf(claims.get("isAdmin").toString()));
	    loginUser.setTenantId(claims.getIssuer());
	    loginUser.setToken(token);
        return loginUser;
    }

    /**
     * 获得userId 如果解析错误返回 -1
     *
     * @param token
     * @return
     */
    public static String getUserIdByToken(String token) {

        Claims claims = null;
        String userid = "-1";
        try {
            claims = Jwts.parser().setSigningKey(getKey(SECRET_KEY, DEFAULTSIGN.getJcaName())).parseClaimsJws(token)
                    .getBody();
            userid = (String) claims.get("id");
        } catch (Exception e) {
            LOGGER.info("解密userId失败：" + token);
            return userid;
        }
        return userid;
    }

    /**
     * 获得username 如果解析错误返回 -1
     *
     * @param token
     * @return
     */
    public static String getUserNameByToken(String token) {

        Claims claims = null;
        String username = "-1";
        try {
            claims = Jwts.parser().setSigningKey(getKey(SECRET_KEY, DEFAULTSIGN.getJcaName())).parseClaimsJws(token)
                    .getBody();
            username = (String) claims.get("username");
        } catch (Exception e) {
            LOGGER.info("解密userId失败：" + token);
            return username;
        }
        return username;
    }


    /**
     * 获得userId 如果解析错误返回 -1
     *
     * @param token
     * @return
     */
    public static String getValueByToken(String token, String value) {
        Claims claims = null;
        try {
            claims = Jwts.parser().setSigningKey(getKey(SECRET_KEY, DEFAULTSIGN.getJcaName())).parseClaimsJws(token)
                    .getBody();
            value = String.valueOf(claims.get(value));
            if (value == null || "".equals(value)) {
                value = "-1";
                return value;
            }
        } catch (Exception e) {
            e.printStackTrace();
            value = "-1";
            return value;
        }
        return value;
    }

    public static void main(String[] args) throws Exception {
    	LoginUser loginUser = new LoginUser();
//        String token = createToken(loginUser);
        String token = "eyJhbGciOiJIUzI1NiJ9.eyJpc3MiOiJDTUJfMTI3MDMxMjQ4NjI0NzIwNjkxMiIsImlkIjozLCJ1c2VybmFtZSI6MTU5MTc1MzQ2NDAwMCwibmlja05hbWUiOiLnjovkvJ_mtpsiLCJwaG9uZSI6IjEzMDUxOTAxMTExIiwiYmFua05hbWUiOiLmtablj5Hpk7booYwiLCJhY2NvdW50Tm8iOiI2MjI4NDgxMTExMTExMTExMTEiLCJ0aW1lU3RhbXAiOjE1OTE4NDI3MzcyMTksImlzQWRtaW4iOjB9.toDSZUDj_QNafa8LO5c7Z_yanexFSa8QmrSVwvJqghs";
        //eyJhbGciOiJIUzI1NiJ9.eyJpc3MiOiJndyIsInVzZXJJZCI6IuW8oOS4iSJ9.M94aWta78LxBlGrWtErQfebwns9ZOobdrtz6qZUg-28
        System.out.println(token);
        LoginUser llll = parserToken(token);
        System.out.println(llll);
//        System.out.println(getValueByToken(token, "enterpriseId"));
//        System.out.println(getUserIdByToken(token));

        Long timeStamp = (Long) parserToken(token).getLoginTime();

        long ts = System.currentTimeMillis();
        if (ts - timeStamp > 1000 * 120) {
            System.out.println("token超时");
        } else {
            System.out.println("token有效");
        }

        //System.out.println(UUID.randomUUID().toString().replace("-", "").substring(0, 16));

    }

}
